/*
The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

Alternatively, the contents of this file may be used under the
terms of the GNU General Public License Version 2 or later (the
"GPL"), in which case the provisions of the GPL are applicable 
instead of those above.

Copyright (C) 2002 Tomas Styblo <tripie@cpan.org>
Copyright (C) 2006 Vladimir Marek <vlmarek@volny.cz>
*/

/*
How to add new preference
=========================

a) Add the name to the prefs hash below. Let's say "new.config.var" : "Char".
   Available types are Bool, Char and Int.
b) At pref-mozex.xul add corresponding UI element. It's ID _must be_
   prefMozexNewConfigVar
c) To obtain preference value in mozex.js or elsewhere, just call
   mozexReadPref("new.config.var");
*/

var prefs =
{
    "debug.exec"           : "Bool" , 
    "intercept.mailto"     : "Bool" , 
    "intercept.news"       : "Bool" , 
    "intercept.telnet"     : "Bool" , 
    "intercept.ssh"        : "Bool" , 
    "intercept.ftp"        : "Bool" , 
    "intercept.irc"        : "Bool" , 
    "general.tmpdir"       : "Char" , 
    "command.mailer"       : "Char" , 
    "command.source"       : "Char" , 
    "command.source.hotkey" : "Char" ,
    "command.textarea"     : "Char" , 
    "command.textarea.utf" : "Bool" , 
    "command.textarea.unwrap" : "Bool" , 
    "command.textarea.hotkey" : "Char" ,
    "command.textarea.file.extension" : "Char" ,
    "command.textarea.file.deleteonrefresh" : "Bool" ,
    "command.news"         : "Char" , 
    "command.telnet"       : "Char" , 
    "command.ssh"          : "Char" , 
    "command.ftp"          : "Char" , 
    "command.irc"          : "Char" , 
    "command.download"     : "Char" , 
    "command.ed2k"         : "Char" , 
    "command.aim"          : "Char" , 
    "universal.schemes"    : "Char" , 
    "universal.command"    : "Char"
}

const mozexConstDirectory = 1;
const mozexConstFile = 2;

var mozex_prefs = Components.classes["@mozilla.org/preferences-service;1"].
    getService(Components.interfaces.nsIPrefService).getBranch("mozex.");

function mozexReadPref(name) {
    if (mozex_prefs.prefHasUserValue(name)) {
        var type = mozex_prefs.getPrefType(name);
        if (type & 128) {
            return mozex_prefs.getBoolPref(name);
        }
        else if (type & 64) {
            return mozex_prefs.getIntPref(name);
        }
        else if (type & 32) {
            return mozex_prefs.getCharPref(name);
        }
        else {
            return null;
        }
    }
    else {
        return null;
    }
}

function name2id(name)
{
    /* "command.editor" -> "prefMozexCommandEditor" */

    var id="prefMozex";
    var array=name.split(/\./);
    for (var atom in array)
    {
        id+=array[atom].slice(0,1).toUpperCase();
        id+=array[atom].slice(1);
    }
    return id;
}

function initMozexPref()
{
    var error_message="";
    for (var x in prefs)
    {
        try
        {
            var id=name2id(x);
            var access = (prefs[x] == "Bool") ? "checked" : "value";
            document.getElementById(id)[access] = mozexReadPref(x);
        }  catch (e) {
            error_message+=e+" id='"+id+"'\n";
        }
    }

    /* Init hotkey display */
    var elem=document.getElementById("prefMozexCommandTextareaHotkeyString");
    elem.value = prefMozexHotnumToString (mozexReadPref('command.textarea.hotkey'));

    elem=document.getElementById("prefMozexCommandSourceHotkeyString");
    elem.value = prefMozexHotnumToString (mozexReadPref('command.source.hotkey'));

    if(error_message.length>0)
    {
        alert("initMozexPref:\n"+error_message);
    }
}


function initMozexPrefPanel()
{
    initMozexPref();
}


/*
    changeMozexPrefs

    Go through the xul and pick all settings storing them into mozilla
    preferences
*/
function changeMozexPrefs()
{
    var error_message="";
    for (var x in prefs)
    {
        try
        {
            var id=name2id(x);
            var access = (prefs[x] == "Bool") ? "checked" : "value";
            mozex_prefs["set"+prefs[x]+"Pref"](x, document.getElementById(id)[access]);
        }
        catch (e)
        {
            error_message+=e+" id='"+id+"'\n";
        }
    }
    if(error_message.length>0)
    {
        alert("changeMozexPrefs:\n"+error_message);
    }
}


/*
    prefMozexHotnumToString

    Convert hotnum into descriptive string, like "Ctrl e"
*/
function prefMozexHotnumToString(str)
{
    var array=str.split(/,/);
    return (array[0] != '0' ? "Ctrl " : "") +
           (array[1] != '0' ? "Alt "  : "") +
           (array[2] != '0' ? "Meta " : "") +
           (array[4] == '0' ? String.fromCharCode(array[3]) : "uknown(keycode="+array[4]+")");
}


/*
    prefMozexHotkeyToHotnum

    Translate keyboardevent to string
    Ctrl+e  = "1,0,0,101,0"
    Alt+F11 = "0,1,0,0,122"

    Those values might be different on Mac
*/
function prefMozexHotkeyToHotnum(e)
{
    return (e.ctrlKey ? '1' : '0') + ',' +
           (e.altKey  ? '1' : '0') + ',' +
           (e.metaKey ? '1' : '0') + ',' +
           e.which + ',' + e.keyCode;
}


/*
    prefMozexHotkeyChange

    id - id value of the hotkey textbox. There has to exist 
        "prefMozexCommand"+id+"Hotkey"       - stores "1,0,0,101,0"
        "prefMozexCommand"+id+"HotkeyString" - stores "Ctrl+e"
    e  - event

    Interpret the key just pressed and set apropriate config value. Hotkey
    consists of five parts (ctrlKey, altKey, metaKey, which, keyCode). We
    translate this into string (see prefMozexHotkeyToHotnum) called hotnum.
    This string is what is stored into preferences.
*/
function prefMozexHotkeyChange(id, e)
{
    try
    {
        e.preventDefault();
        var tmp = prefMozexHotkeyToHotnum (e);
        document.getElementById("prefMozexCommand"+id+"Hotkey").value=tmp;
        document.getElementById("prefMozexCommand"+id+"HotkeyString").value =
            prefMozexHotnumToString (tmp);
        changeMozexPrefs();
    }
    catch(err)
    {
        alert("prefMozexHotkeyChange: " + err);
    }
}


/*
    prefMozexHotkeyDisable

    Set the hotkey to empty string, effectively disabling it
*/
function prefMozexHotkeyDisable(id)
{
    document.getElementById("prefMozexCommand"+id+"Hotkey").value="";
    document.getElementById("prefMozexCommand"+id+"HotkeyString").value="";
    changeMozexPrefs();
}

/*
   prefMozexBrowse

   Function calls FilePicker. If successful, it changes given textarea

   id   - id of the textarea to be changed
   type - mozexConstFile or mozexConstDirectory
*/
function prefMozexBrowse(id, type)
{
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                .createInstance(nsIFilePicker);
    fp.init(window,
            "Select a "+(type == mozexConstFile ? "File" : "Directory"),
            (type == mozexConstFile ? nsIFilePicker.modeOpen : nsIFilePicker.modeGetFolder));
    var res = fp.show();

    if (res == nsIFilePicker.returnOK)
    {
      var file = fp.file.path;
      if (type == mozexConstFile && file.match(/ /) != null)
      {
          file="\"" + file + "\"";
      }
      document.getElementById(id).value = file;
    }
}

/*
vim: tabstop=4:shiftwidth=4:expandtab:cindent
*/
